;-------------------------------------------------------------------------;
;
;		Project:	count down timer with 4 display 7-segment
;    	Filename:	t2f8426.asm
;    	Date:		30/01/2007                             
;    	Author:		nferrarese@inwind.it
;    	Company:	http://spazioinwind.libero.it/nferrarese/
;
;
;		History:	2.6 30/01/2007 fixed little bug key routine 
;					2.5 14/07/2003 first release web page
;
;-------------------------------------------------------------------------;
;
;			SCHEMA ELETTRICO
;    	http://spazioinwind.libero.it/nferrarese/
;
;-------------------------------------------------------------------------;
;			PROGRAMMA 
;-------------------------------------------------------------------------;
        PROCESSOR       16F84
        RADIX           DEC
        INCLUDE         "P16F84.INC"
        ERRORLEVEL      -302

        ;Setup of PIC configuration flags
        ;XT oscillator
        ;Disable watch dog timer
        ;Enable power up timer
        ;Disable code protect

        __CONFIG        3FF1H

;-------------------------------------------------------------------------;
; 		definizione costanti simboliche
;-------------------------------------------------------------------------;
	; PORTA output 
dgt0	EQU	0		
dgt1	EQU	1
dgt2	EQU	2
dgt3	EQU	3
out	EQU	4

	; PORTB input 
psctrl	EQU	0		; ingresso power supply control
;	EQU	1		;
;	EQU	2		;
;	EQU	3		;
;	EQU	4		;
select	EQU	5		; P3
mode	EQU	6		; P2 ingresso mode push button
go	EQU	7		; P1 ingresso go push button

	; PORTB output
;a	EQU	0
;b	EQU	1
;c	EQU	2
;d	EQU	3
;e	EQU	4
;f	EQU	5
;g	EQU	6
dot	EQU	7		; dot dei display

swtchin	EQU	11100001b	; costante per lo swicth input linee
fuse	EQU	235		; teorico 239, mah...
lngprss	EQU	60		; long press 
spdst	EQU	5		; speed set

	; rnflg register bit flag
adj	EQU	0		; adjust bit flag
hm	EQU	1		; hhmm or mmss bit flag	 (1 hhmm, 0 mmss)
tmrun	EQU	2		; timer run bit flag
tmrp	EQU	3		; timer paused bit flag
onesec	EQU	4		; bit che cambia stato ogni secondo (2Hz)
halfsec EQU	5		; bit che cambia stato ogni 0.5 sec (1Hz)
setmd	EQU	6		; set mode bit flag
alarm	EQU	7		; alarm bit flag

;-------------------------------------------------------------------------;
; 	Origine dati memoria. Riserva spazio per registri
;-------------------------------------------------------------------------;
	ORG	0Ch
sec	RES	1	; this register holds the value of seconds
sec10   RES	1	; holds value of 10's of seconds
min	RES	1	; holds value of minutes
min10	RES	1	; holds value of 10's of minutes
hour	RES	1	; holds value of hours
hour10	RES	1	; holds value of 10's of hours
rnflg	RES	1	; run flags register
key	RES	1 	; registro che memorizza lo stato dei tasti
keycnt	RES	1	; key counter
dgtcnt	RES	1	; 8 bit counter, only 2 lowest bits actually used
temp	RES	1	; temporary register
tempa	RES	1	; temporary PORTA
tempb	RES	2	; temporary PORTB
lcnt	RES	1	; loop counter
intcnt	RES	1	; interrupt counter
offset	RES	1	; 
flash	RES	1	; registro maschera per il lampeggio dgt e dot
cntmsec	RES	1	; used in timing of milliseconds
w_tmp	RES	1
s_tmp	RES	1
fsr_tmp	RES	1

;-------------------------------------------------------------------------;
; 				Definizioni
;-------------------------------------------------------------------------;
#define	out_on	bcf	PORTA,out
#define	out_off	bsf	PORTA,out

#define	bank0	bcf	STATUS,RP0 
#define	bank1	bsf	STATUS,RP0 


;-------------------------------------------------------------------------;
;    Here we set up the initial values of the digits in data EEPROM       ;
;-------------------------------------------------------------------------;
	ORG H'2100'
	DE 0		; offset
	DE 0, 0, 1, 0	; dgt0,dgt1,dgt2,dgt3
	DE 0, 0, 2, 0
	DE 0, 0, 3, 0
	DE 0, 0, 4, 0
	DE 0, 0, 5, 0

;-------------------------------------------------------------------------;
;			Reset Vector
;	Punto di inizio del programma al reset della CPU
;-------------------------------------------------------------------------;
	ORG     00H
	goto    init
 
;-------------------------------------------------------------------------;
;			 Interrupt vector
;		Start point for every interrupt handler
;-------------------------------------------------------------------------;
        ORG     04H
	btfss	INTCON,T0IF	; Interrupt generato da TMR0 overflow ?
	goto	rb0ih		; no, vai al rb0/int interrupt handler
tmr0ih				; tmr0interrupt handler
	movwf	w_tmp		; save W
	swapf	STATUS,W	; save status
	movwf	s_tmp		; without changing flags
	movf	FSR,w		;
	movwf	fsr_tmp		;

	btfss	rnflg,adj	; occorre aggiustare tempo ?
	goto	noadj		; no, continua senza aggiustamento.
	nop			; s, lascio passare 16us dall'interrupt
	nop			; prima della scrittura su tmr0
	nop			;
adjust	movlw	fuse		;

	movwf	TMR0		; Scrittura su TMR0.Reset Prescaler.
	bcf	rnflg,adj	; Clear di adjust bit flag
	goto	restore		; Poi ripristina e ritorna.

noadj	decfsz	intcnt,f	; Sono trascorsi 122 interrupt ?
	goto	refresh		; no, esegui solo refresh display
	movlw	122		; s, allora reinizializza
	movwf	intcnt		; interrupt counter	
	call	evryhs		; quindi chiama routine every half second
	bsf	rnflg,adj	; e setta bit flag adjust.Poi refresh display.	

refresh	incf	dgtcnt,f	; incremento il registro
        movlw	00000011b	; digit counter (dgtcnt)
        andwf 	dgtcnt,f	; in modulo 4

	call	getsegm		; mette in tempb codice per i segmenti a..g
	call	bjtmux		; mette in tempa codice per multiplex bjt
	call	refrdot		; gestione del dot
	call	flshdgt		; gestice lampeggio dei dgt
	call	chsegm		; eventuale permuta dei segmenti g e dot

	movlw	11110000b	; spegne i 4 bjt del display
	andwf	PORTA,f		; senza alterare altre uscite di PORTA
	call	keyin		; legge linee input
outdgt	movf	tempb,w		; output del segment pattern
	movwf 	PORTB		; sulla portb

outbjt	movf	tempa,w		; In tempa c' il nuovo bjt pattern
	movwf 	PORTA		;

restore				; ripristina i registri di lavoro
	movf	fsr_tmp,w	; all'uscita dall' interrupt handler
	movwf	FSR
	swapf	s_tmp,w		; get original status back
	movwf	STATUS		; into status register
	swapf	s_tmp,f		; old no flags trick again
	swapf	s_tmp,w		; to restore W
	bcf	INTCON,T0IF	; resetta interrupt flag
	retfie			; ritorna con GIE abilitato

rb0ih	nop			; rb0/int interrupt handler
	nop			; esegue questo handler al risveglio
	nop			; dallo stato sleep.
	nop			; Il fronte di salita sul pin RB0/INT dovuto
	nop			; al ritorno dell'energia elettrica,
	nop			; ha causato l'interrupt.
	bank1			; attiva banco1
	clrf	TRISB		; ripristino linea RB0 come uscita
	bank0			; ritorna al banco0
	clrf	PORTB		;
	movlw	00010000b	; spegne bjt dgt e uscita
	movwf	PORTA		;
	movlw	250		;
	call	nmsec		;
	out_off			; Ipotizzo uscita fosse disattiva
	call	dly20		; prima del black-out
	btfsc	rnflg,alarm	; Il timer ha terminato il conteggio ?
	goto	$+3		; s,lascia uscita bassa e ritorna
	btfsc 	rnflg,tmrun	; no.Timer attivo ?
	out_on			; s, allora uscita deve stare on
	bcf	INTCON,INTF	; reset interrupt flag di RB0/INT
	bcf	INTCON,INTE	; disabilita RB0/INT interrupt	
	retfie

;-------------------------------------------------------------------------;
;			 Main body							
;-------------------------------------------------------------------------;
init	bank1			; attiva banco1
        movlw   00000000b	; 0 uscita , 1 ingresso
        movwf   TRISA
        movwf   TRISB

	; RB[0-7] pull-up disabilitate
	; TMR0 clock interno, prescaler assegnato a TMR0
	; prescaler rate 1:16, RB0/INT interrupt on rising edge
	movlw	11010011b	
	movwf	OPTION_REG

        bank0			; attiva banco0
	movlw	122		; inizializza registro
	movwf	intcnt		; interrupt counter
	clrf	dgtcnt		; inizilalizza digit counter
	clrf	offset		;
	call	ready		; prepara timer a partire
	bsf	INTCON,GIE	; Abilita Global interrupt
	bsf	INTCON,T0IE	; Abilita TMR0 interrupt

keychk		
	btfsc	key,go		; check for go pressed	
	call	go_sr		; go service routine
	btfsc	key,mode	; check for mode pressed	
	call	mode_sr		; mode service routine
	btfsc	key,select	; check for select pressed	
	call	slct_sr		; select service routine
	btfsc	key,psctrl	; check power supply	
	call	blckout		;	
        goto	keychk		;

;-------------------------------------------------------------------------;
;	      Prepara timer al conto alla rovescia
;-------------------------------------------------------------------------;
ready	out_off			; spegne bjt in uscita
	clrf	key		; azzera registro key
	clrf	rnflg		; azzera bits flag del registro runflag
	clrf	flash		; nessun lampeggio
	call	tmreset		; azzera registri tempo
	call	getee		; richiama valori dalla eeprom
	return  

;-------------------------------------------------------------------------;
;  Questa routine riempie i registri del display con i dati della eeprom  ;
;-------------------------------------------------------------------------;
getee  	movlw	0		; 
	call	readee		; read eeprom location 0
	movwf	offset		; and hold value in offset

	movlw	h'11'		; point to
	movwf	FSR		; hour10 file register	
	bsf	rnflg,hm	; ipotizzo modo hhmm

	movf	offset,w	; Leggo il contenuto della 4a cella eeprom
	addlw	4		; successiva a offset
        call	readee		; 
	andlw	128		;
 	btfss	STATUS,Z	; modo mmss in eeprom ?		
	goto	fillreg		; no
	bcf	rnflg,hm	; s,allora bit hm=0 e  
	movlw	2		; sottraggo literal 2 dal puntatore FSR
 	subwf	FSR,f		; in modo da puntare al registro min10
				; e non al registro hour10 come ipotizzato.
	
fillreg	movlw	4		; inizializzo loop counter
	movwf	lcnt		; per compiere 4 iterazioni

eerloop	movf	offset,w	; carica in w  l'indirizzo cella eeprom
	addwf	lcnt,w		; aggiungendo offset
	call	readee		; legge cella.Il valore letto  in w
	movwf	INDF		; copio valore di W in registro puntato da FSR
	decf	FSR,f		; muove puntatore
	decfsz	lcnt,f		; decrementa loop counter.E'arrivato a 0 ?
	goto 	eerloop		; no, continua leggere epprom 	
	return			; s, allora ritorna.

;-------------------------------------------------------------------------;
;     Questa routine legge un byte dalla eeprom.
;     Richiede in ingresso indirrizzo EEPROM d leggere in W
;     Restituisce valore letto in W.
;-------------------------------------------------------------------------;
readee	movwf	EEADR		; set up eeprom address from W
        bank1			; change to page 1
        bsf	EECON1,RD	; set the read bit
        bank0			; back to page 0
        movf 	EEDATA,W	;
        return          

;-------------------------------------------------------------------------;
;         Questa routine azzera tutti e 6 i registri del display	  ;
;-------------------------------------------------------------------------;
tmreset
	movlw	0Ch		; inizializzo puntatore
	movwf	FSR		; 
	
	movlw	6		; inizializza loop counter
	movwf	lcnt		; per compiere 6 iterazioni

zloop	clrf	INDF		; azzera file register puntato da FSR
	incf	FSR,f		; continua incrementando puntatore
	decfsz	lcnt,f		; fino a quando il loop counter
	goto	zloop		; arriva a zero.
	return			; Poi ritorna.

;-------------------------------------------------------------------------;
;  Questa routine mette in tempb il codice per i segmenti a..g
;-------------------------------------------------------------------------;
getsegm	movf	dgtcnt,w	; copia digit counter in w
	addlw   0Ch             ; inizializza puntatore		
	btfsc	rnflg,hm	; aggiunge offset a puntatore
	addlw	2 		; se modo hhmm
	movwf	FSR		; Punta a cella da rinfrescare.
	movf	INDF,w		; copia il valore del registro puntato in w
	call	segmpat		; chiama tabella segment pattern
	movwf	tempb		; tempb contiene codice per i segmenti a..g
	return

;-------------------------------------------------------------------------;
;   Questa tabella restituisce il bit pattern per i segmenti, per dato
;          	 un dato valore in w compreso tra 0 e 31			
;-------------------------------------------------------------------------;
segmpat	andlw	00011111b	; considera solo primi 5 bit
	ADDWF 	PCL,f		; metti in w il bit pattern per i bjt
	retlw	11000000b	; 0
	retlw	11111001b	; 1
	retlw	10100100b	; 2
	retlw	10110000b	; 3
	retlw	10011001b	; 4
	retlw	10010010b	; 5
	retlw	10000010b	; 6
	retlw	11111000b	; 7
	retlw	10000000b	; 8
	retlw	10010000b	; 9
	retlw	10001000b	; A	10
	retlw	10000011b	; b	11
	retlw	11000110b	; C	12
	retlw	10100001b	; d	13
	retlw	10000110b	; E	14
	retlw	10001110b	; F	15
	retlw	11111111b	; null	16
	retlw	10001001b	; H	17
	retlw	11000111b	; L	18
	retlw	10001100b	; P	19
	retlw	11000001b	; U	20
	retlw	10011111b	; |	21
	retlw	10111111b	; -	22
	retlw	10011100b	; 	23
	retlw	10100011b	; o	24
	retlw	11000110b	; [	25
	retlw	11110000b	; ]	26
	retlw	10000110b	; E	27
	retlw	10000110b	; E	28
	retlw	10000110b	; E	29
	retlw	10000110b	; E	30
	retlw	10000110b	; E	31	

;-------------------------------------------------------------------------;
;          Questa rotuine gestice lo scambio dei segmenti g e dot
;            per il dgt1 che   montato al rovescio					
;-------------------------------------------------------------------------;
chsegm
	movf	dgtcnt,w	; 
	sublw	1		;  il turno
	btfss	STATUS,Z	; del dgt1 ?
	return			; no,ritorna
	movf	tempb,w		; s,copia tempb
	movwf	tempb+1		; in tempb+1.
	andlw	00111111b	; Azzera 2 bit linee g e dot
	movwf	tempb		; del registro tempb.	
	btfsc	tempb+1,7	; Il bit7 di tempb era settato ?	
	bsf	tempb,6		; s, allora setta il bit6
				; no, lascia bit6 a 0.
	btfsc	tempb+1,6	; Il bit6 di tempb era settato ?	
	bsf	tempb,7		; s, allora setta il bit7
	return			; no, lascia bit7 a 0 e ritorna.

;-------------------------------------------------------------------------;
;      Questa rotuine gestice l'attivazione in multiplexer dei bjt					
;-------------------------------------------------------------------------;
bjtmux	movf	PORTA,w		; Mette valore corrente PORTA in W
	andlw	11110000b	; maschera con 1 i bit da lasciare inalterati
	movwf	tempa		; salva in tempa
	movf	dgtcnt,w	; recall digit #
	call 	bjtpat		; get bjt pattern
	andlw	00001111b	; maschera con 1 i bit da sovrascrivere
	iorwf	tempa,f		; intarsia vecchio valore con nuovo in tempa
	return				


;-------------------------------------------------------------------------;
;    Questa tabella restituisce il bit pattern per accendere uno dei bjt  ;
;-------------------------------------------------------------------------;
bjtpat	ADDWF 	PCL,f		; metti in w il bit pattern per i bjt
        RETLW 	00000001b	; a low, (0), turns the transistor on  
        RETLW 	00000010b                  
        RETLW 	00000100b                  
        RETLW 	00001000b
	return

;-------------------------------------------------------------------------;
; 		Questa routine gestice il flash del display
; 		sulla base del contenuto del registro flash			
;-------------------------------------------------------------------------;
flshdgt				; gestione del flash per i digit
	btfsc	rnflg,halfsec	; s.Semiperiodo con i display spenti ?
	return			; no, semiperiodo con dgt accesi
	movf	dgtcnt,w	; s,copia digit counter in w
	btfss	STATUS,Z	;  il turno di dgt0 ?
	goto	flhdgt1		; no, vai a vedere se dgt2 in flash
	btfsc	flash,dgt0	; s,e dgt0 deve lampeggiare ?	
	bcf	tempa,dgt0	; s, allora spegni Q0
	return 			; in ogni caso poi ritorna

flhdgt1	movf	dgtcnt,w	; copia digit counter in w
	sublw	1
	btfss	STATUS,Z	;  il turno di dgt1 ?
	goto	flhdgt2		; no, vai a vedere se dgt2 in flash
	btfsc	flash,dgt1	; s,e dgt1 deve lampeggiare ?	
	call	dispoff		; s, allora spegni segmenti a..g
	return 			; in ogni caso poi ritorna

flhdgt2	movf	dgtcnt,w	; s,copia digit counter in w
	sublw	2
	btfss	STATUS,Z	;  il turno di dgt2 ?
	goto	flhdgt3		; no, vai a vedere se dgt2 in flash
	btfsc	flash,dgt2	; s,e dgt2 deve lampeggiare ?	
	call	dispoff		; s, allora spegni segmenti a..g
	return 			; e in ogni caso poi ritorna

flhdgt3				;  turno di dgt3
	btfsc	flash,dgt3	; dgt3 deve lampeggiare ?	
	bcf	tempa,dgt3	; s, allora spegni Q3
	return 			; e in ogni caso poi ritorna
	
;-------------------------------------------------------------------------;
; 	Questa rotuine mette in tempb il codice per tenere
;	 i degmenti a..g spenti. Dot inalterato		
;-------------------------------------------------------------------------;
dispoff
	movlw	01111111b	;
	iorwf	tempb,f		;
	return

;-------------------------------------------------------------------------;
; 		Questa routine gestice il flash del dot
; 	         	se e quando occorre			
;-------------------------------------------------------------------------;
refrdot				; Rinfresca dot quando solo se occorre
	btfss	rnflg,hm	; Timer modalit hhmm ?
	goto	dotoff		; no,allora dot deve stare spento
	movf	dgtcnt,w	; vado a vedere se  il turno
	movwf	temp		; di dgt0 o dgt3 facendo l'exor
	rrf	temp,f		; dei bit0 e bit1 del registro
	xorwf	temp,w		; digit counter.
	andlw	00000001b
	btfsc	STATUS,Z	; turno di dgt0 o dgt3 ?	
	goto	dotoff		; s, allora dot deve stare spento
				; no,allora  in corso refresh di dgt1 o dgt2
dothm	btfss	flash,dot	; il dot deve lampeggiare ?
	goto	dotstd		; no, il flash per il dot non  attivo
	btfss	rnflg,halfsec	; s,il dot ha flash attivo.Semiperiodo on ?	
	goto	dotoff		; no, allora dot spento
	goto	doton		; s, accendi dot

dotstd	btfss	rnflg,tmrun	; Il timer  attivo ?	
	goto	doton		; no, timer in hhmm non attivo
	btfss	rnflg,halfsec	; s. Semiperiodo con display accesi ?	
	goto	dotoff		; no, allora dot spento
				; s, allora dot acceso.
doton	bcf	tempb,dot	; accende segmento dot
	goto	$+2		; spegne segmento dot

dotoff	bsf	tempb,dot
	return

;-------------------------------------------------------------------------;
;      Questa routine cambia le linee RB6 eRB7 in ingresso,
;	 ne legge i valori,e le  ripristina come uscite			
;-------------------------------------------------------------------------;
keyin	bank1			; change to page 1
	movlw   swtchin		; cambia in ingressi 
        movwf	TRISB		; le line psctrl,go,mode
        bank0			; change to page 0
	clrf	key		; presumo ingressi tutti inattivi
	nop			; aspetta
	nop			; si stabilizzino
	nop			; le linee
	nop			;
check5V	btfsc	PORTB,psctrl	; C' la tensione 5V ?
	goto	checkgo		; s, continua
	bsf	key,psctrl	; no, setta corrispondente nel reg key
	goto	restout		; quindi ripristina e ritorna
checkgo	btfss	PORTB,go	; tasto go premuto ?
	goto	checkmd		; no, continua con check mode
	bsf	key,go		; s, setta bit corrispondente in key
	goto	restout		; quindi ripristina e ritorna
checkmd	btfss	PORTB,mode	; tasto mode premuto ?
	goto	checksl		; no, continua con check select 
	bsf	key,mode	; s, setta bit corrispondente in key
	goto	restout		; quindi ripristina e ritorna
checksl	btfss	PORTB,select	; tasto select premuto ?
	goto	restout		; no,ripristina e ritorna
	bsf	key,select	; s, setta bit corrispondente in key
				; quindi ripristina e ritorna
restout	bank1			; change to page 1
	movlw	0		; ripristina PORTB
        movwf	TRISB		; in tutte uscite
        bank0			; change to page 0
	return

;-------------------------------------------------------------------------;
;         	Routine eseguita mezzo secondo			          ;
;-------------------------------------------------------------------------;
evryhs	movlw	1<<halfsec	; ogni mezzo secono
	xorwf	rnflg,f		; inverti bit flag halsec
	btfsc	rnflg,halfsec	;  trascorso un secondo ?
	call	evrys		; s, chiama routine every second	
	return			; no, ritorna con solo bit halfsec negato

;-------------------------------------------------------------------------;
;         	Routine eseguita ogni secondo			          ;
;-------------------------------------------------------------------------;
evrys	movlw	1<<onesec	; ogni secondo
	xorwf	rnflg,f		; inverti bit flag onesec
	btfss	rnflg,tmrun	; timer run ?
	return			; no, return
	btfss	rnflg,tmrp	; s, il timer  in pausa ?
	call	chngsec		; no, allora cambia i secondi
	return			; s,ritorna con bit flag onesec negato

;-------------------------------------------------------------------------;
;        Routine che decrementa di 1 sec il contenuto dei registri timer  ;
;-------------------------------------------------------------------------;
chngsec				; change second
chkmd	call	hmtoms		; verifica il cambio da hhmm a mmss
dectime	decf	sec,f		; decrementa i secondi
	incfsz	sec,w		; c' underflow ?
	goto	chkz		; no.Vai a controllore se tempo  0
	movlw	9		; s, riparti con
	movwf	sec		; i secondi a 9 quindi
	decf	sec10,f		; decrementa le decine di secondi
	incfsz	sec10,w		; e controlla se c' underflow
	return			; no, ritorna

	movlw	5
	movwf	sec10
	decf	min,f
	incfsz	min,w
	goto	chkz

	movlw	9
	movwf	min
	decf	min10,f
	incfsz	min10,w
	goto	chkz		

	movlw	5
	movwf	min10
	decf	hour,f
	incfsz	hour,w
	goto	chkz

	movlw	9
	movwf	hour
	decf	hour10,f

chkz	call	chkzero
	btfss	STATUS,Z	; tutti registri tempo sono 0 ?
	return			; no.
	call	alarm_on	; s,attiva alarm
	return

;-------------------------------------------------------------------------;
;          Questa routine restituisce W=0 se tutti i registri
;			tempo del timer sono 0			
;-------------------------------------------------------------------------;
chkzero	movf	hour10,w	; controlla se timer arrivato a zero
	andlw	00001111b	; 
	iorwf	hour,w		; 
	iorwf	min10,w		; 
	iorwf	min,w		;
	iorwf	sec10,w		;
	iorwf	sec,w		;
	return

;-------------------------------------------------------------------------;
;          Questa routine gestisce la pressione del tasto "go"		  ;
;-------------------------------------------------------------------------;				
go_sr	call	dly20		; aspetta 20msec
	btfss	key,go		; tasto ancora premuto ?
	return			; no, allora  un falso allarme ritorna.
	call	dly20		; aspetta 20msec
	btfss	key,go		; 2 controllo: tasto ancora premuto ?
	return			; no, falso allarme ritorna.
				; s, allora entra in routine di servizio
	btfsc	rnflg,alarm	; Il timer ha finito il conteggio ?
	goto	wgo		; s, allora ritorna senza fare nulla.
	btfsc	rnflg,setmd	; no.Timer in modalit set time mode ?
	goto	setdspl		; s, vai settare il display
	btfss	rnflg,tmrun	; no.Il timer  attivo ?
	goto	startmr		; no, allora start del timer
	btfss	rnflg,tmrp	; s,timer attivo.E' in pausa ?
	goto	wgo		; no,timer attivo non in pausa: ritorna.
	bcf	rnflg,tmrp	; s, allora resetta timer paused bit flag
	clrf	flash		; smetti di far lampeggiare il display
	goto	wgo		; e ritorna

startmr	bsf	rnflg,tmrun	; abilita timer run flag
	out_on			; attiva bjt in uscita
	bsf	rnflg,halfsec	; sincronizzo partenza
	movlw	122		; count down in modo che conteggio
	movwf	intcnt		; parta con un sec pieno
	goto	wgo		; e ritorna

setdspl	movf	flash,w		; verifica se  in corso la scelta 
	andlw	00001111b	; per la modalit hm, andando a vedere se
	sublw	00001111b	; stanno lampeggiando  tutti e 4 i dgt.
	btfss	STATUS,Z	; E' in corso il set del bit hm ?
	goto	chndgt		; no,  in corso il set di un digit

chnghm	bcf	flash,dot	; s, allora fai smettere di lampeggiare dot
	movlw	1<<hm		; quindi nega il bit hm nell registro rnflg
	xorwf	rnflg,f		; senza alterare gli altri bit
	goto	wgo		; e ritorna.

chndgt	movf	flash,w		; verifico se  in corso il set dei dgt[0-1]
	sublw	00000011b	; andando a vedere il registro flash
	btfss	STATUS,Z	; set di dgt[0-1] hm in corso ?
	goto	idgt23		; no, prosegui con set dgt[2-3]
idgt01	incf	INDF,f		; s. allora incrementa dgt0
	movf	INDF,w		; metti dgt0 in w
	sublw	10		; sottrai 10
	btfss	STATUS,Z	; dgt0=10 ?
	goto	$+9		; no
	clrf	INDF		; s, allora azzera dgt0
	incf	FSR,f		; punta dgt1
	incf	INDF,f		; incrementa dgt1
	movf	INDF,w		; verifico se dgt1 
	sublw	6		;  in overflow.
	btfsc	STATUS,Z	; dgt1=6 ?
	clrf	INDF		; s, allora zzera dgt1
	decf	FSR,f		; in ogni caso ritorna a puntare dgt0
				; controllo se entro in conteggio veloce
	clrf	keycnt		; resetta key counter
golng10	call	dly20		; aspetta 20 msec
	btfss	key,go		; go ancora premuto ?
	goto	wgo		; no, vai ad aspettare definitivo rilascio.
	incf	keycnt,f	; s, allora incrementa key counter.
	movf	keycnt,w	; confronta il valore di key counter
	sublw	lngprss		; con la costante lngprss
	btfsc	STATUS,C	; keycnt > lngprss ?
	goto	golng10		; no, continua nel go long press loop
				; s, entra in fast go
	clrf	keycnt		; azzera key counter
	movlw	11111100b	; durante conteggio veloce dgt[0-1]	
	andwf	flash,f		; non lampeggiano	
fstgo01	incf	INDF,f		; incrementa dgt0
	movf	INDF,w		; metti dgt0 in w
	sublw	10		; sottrai 10
	btfss	STATUS,Z	; dgt0=10 ?
	goto	$+9		; no
	clrf	INDF		; s, allora azzera dgt0
	incf	FSR,f		; poi punta dgt1
	incf	INDF,f		; e increment dgt1.
	movf	INDF,w		; verifico se dgt1
	sublw	6		;  andato in overflow.
	btfsc	STATUS,Z	; dgt1=6 ?
	clrf	INDF		; s, allora azzera dgt1
	decf	FSR,f		; in ogni caso ritorna a puntare su dgt0

	btfsc	key,go		; controlla go ancora premuto ?
	goto	$+4		; s, allora continua il fast go dgt[0-1] loop
	movlw	00000011b	; no, allora riprendi a far
	iorwf	flash,f		; lampeggiare dgt[0-1]
	goto	wgo		; e poi vai ad aspettare definitivo rilascio.
				; inc.v eloce di dgt0 ogni volta che keycnt>spdt
	call	dly20		; aspetta 20ms
	incf	keycnt,f	; incrementa keycnt
	movf	keycnt,w	; confronta il valore di key counter
	sublw	spdst		; con la costante spdst
	btfsc	STATUS,C	; keycnt > spdst ?
	goto	$-10		; no, continua testare tasto
	clrf	keycnt		; s, allora azzera key counter	
	goto	fstgo01		; e incrementa dgt0 in modo veloce.

idgt23	movf	flash,w		; set 
	sublw	00001100b	; di dgt[2-3]
	btfss	STATUS,Z	; in corso ?
	goto	wgo	   	; no, vai ad aspettare rilascio di go.
	incf	INDF,f		; s. allora incrementa dgt2 (INDF+2 in mode_sr)
	movf	INDF,w		; dgt2 in w
	sublw	10		; sottrai 10
	btfss	STATUS,Z	; dgt2=10 ?
	goto	$+9		; no
	clrf	INDF		; s, allora azzera dgt2
	incf	FSR,f		; poi punta dgt3
	incf	INDF,f		; incrementa dgt3
	movf	INDF,w		; controllo se dgt3 
	sublw	10		; in overflow ?
	btfsc	STATUS,Z	; dgt3=10 ?
	clrf	INDF		; s, allora azzera dgt3
	decf	FSR,f		; in ogni caso ritorna a puntare dgt2

	clrf	keycnt		; resetta key counter
golng23	call	dly20		; aspetta 20 msec
	btfss	key,go		; go ancora premuto ?
	goto	wgo		; no, vai ad aspettare definitivo rilascio.
	incf	keycnt,f	; s, allora incrementa key counter.
	movf	keycnt,w	; confronta il valore di key counter
	sublw	lngprss		; con la costante lngprss
	btfsc	STATUS,C	; keycnt > lngprss ?
	goto	golng23		; no, continua nel go long press loop
				; s, entra in fast go
	clrf	keycnt		; azzera key counter
	movlw	11110011b	; smetti di far lampeggiare dgt[2-3]	
	andwf	flash,f		;	
fstgo23	incf	INDF,f		; incrementa dgt2
	movf	INDF,w		; metti dgt2 in w
	sublw	10		; sottrai 10
	btfss	STATUS,Z	; dgt2=10 ?
	goto	$+9		; no
	clrf	INDF		; s, allora azzera dgt2
	incf	FSR,f		; poi punta dgt3
	incf	INDF,f		; incrementa dgt3
	movf	INDF,w		; controlloa se dgt3
	sublw	10		; in overflow.
	btfsc	STATUS,Z	; dgt3=10 ?
	clrf	INDF		; s, allora azzera dgt3
	decf	FSR,f		; in ogni caso torna a puntare su dgt2

	btfsc	key,go		; controlla go ancora premuto ?
	goto	$+4		; s, allora continua il fast go loop
	movlw	00001100b	; no, riprendi a far
	iorwf	flash,f		; lampeggiare dgt[0-1]
	goto	wgo		; e vai ad aspettare definitivo rilascio.
	call	dly20		; aspetta 20ms
	incf	keycnt,f	; incrementa keycnt
	movf	keycnt,w	; confronta il valore di key counter
	sublw	spdst		; con la costante speed set 
	btfsc	STATUS,C	; keycnt > spdst ?
	goto	$-10		; no, continua testare tasto
	clrf	keycnt		; s, allora azzera key counter
	goto	fstgo23		; e continua con fast go dgt[2-3] loop

wgo	btfsc	key,go		; attendi il rilascio
	goto	wgo		; non ancora rilasciato
	call 	dly20		; aspetta 20msec
	btfsc	key,go		; 2 controllo
	goto	wgo		; continua a controllare
	return
	
;-------------------------------------------------------------------------;
;          Questa routine gestisce la pressione del tasto "mode"	  ;
;-------------------------------------------------------------------------;
mode_sr call	dly20		; aspetta 20msec
	btfss	key,mode	; tasto ancora premuto ?
	return			; no, falso allarme ritorna.
	call	dly20		; aspetta 20msec
	btfss	key,mode	; 2 controllo: tasto ancora premuto ?
	return			; no, falso allarme ritorna.
				; s, allora entra in routine di servizio
	btfsc	rnflg,alarm	; timer con alarm attivo ?
	goto	restart		; s, resetta timer e torna	
	btfsc	rnflg,setmd	; no. Il timer  in set mode ?
	goto	stime		; s, vai a set time
	btfss	rnflg,tmrun	; no. Il  timer attivo ?
	goto	sstime		; no, allora vai a start set time
	btfss	rnflg,tmrp	; s.Timer in pausa ?
	goto	pause		; no, allora metticelo
restart	call	ready		; s, allora ripristina il tempo
	goto	wmode		; aspetta rilascio tasto,poi ritorna.
			
pause	bsf	rnflg,tmrp	; abilita bit flag timer paused
	movlw	10001111b	; fai lampeggiare tutti i dgt
	movwf	flash
	goto	wmode		; aspetta rilascio tasto,poi ritorna.
	
sstime	bsf	rnflg,setmd	; abilito set mode bit flag
	bsf	rnflg,hm	; ipotizzo modalit hhmm di default
	call	tmreset		; azzero display
	movlw	10001111b	; fai lampeggiare tutti i dgt e dot
	movwf	flash
	goto	wmode		; aspetta rilascio tasto,poi ritorna.
stime 	movf	flash,w		;  stato premuto
	andlw	00001111b	; il tasto mode
	sublw	00001111b	; mentre lampeggiano 
	btfss	STATUS,Z	; tutti dgt ?
	goto	$+8		; no, allora prosegui
	movlw	h'0C'		; s.Inizializza il puntatore	
	btfsc	rnflg,hm	;  stata scelta modalit mmss ?
	addlw	2		; no aggiungi offset
	movwf	FSR		; carica il puntatore
stdgt01	movlw	00000011b	; fai lampeggiare 
	movwf	flash		; solamente dgt[0-1]
	goto	wmode		; aspetta rilascio tasto,poi ritorna.
	movf	flash,w		;  stato premuto 
	sublw	00000011b	; il tasto mode
	btfss	STATUS,Z	; mentre lampeggiano i  dgt[0-1] ?
	goto	$+6		; no, allora prosegui
stdgt23	movlw	00001100b	; s,allora fai lampeggiare
	movwf	flash		; solamente dgt[2-3]
	movlw	2		; punta
	addwf	FSR,f		; 
	goto	wmode		; aspetta rilascio tasto,poi ritorna.

	call	chkzero		; E'terminato set time.	
	btfsc	STATUS,Z	; Il tempo inserito  uguale a zero ?
	goto	sstime		; s, allora non va bene, ricomincia
	clrf	flash		; no, allora disabilita ogni lampeggio
	bcf	rnflg,setmd	; resetta set mode bit flag
	call	hmtoms		; verifica se moho hm o ms	
svtime	bcf	INTCON,GIE	; disabilita tutti interrupts
	call	putee		; salva nella eeprom
	call	dly20		; breve spegnimento delle cifre x conferma
	bsf	INTCON,GIE	; riabilita interrupts

wmode	btfsc	key,mode	; attendi il rilascio
	goto	wmode		; non ancora rilasciato
	call 	dly20		; aspetta 20msec
	btfsc	key,mode	; 2 controllo
	goto	wmode		; continua a controllare
	return


;-------------------------------------------------------------------------;
;          Questa routine gestisce la pressione del tasto "select"	  ;
;-------------------------------------------------------------------------;
slct_sr	call	dly20		; aspetta 20msec
	btfss	key,select	; tasto ancora premuto ?
	return			; no, allora  un falso allarme ritorna.
	call	dly20		; aspetta 20msec
	btfss	key,select	; 2 controllo: tasto ancora premuto ?
	return			; no, falso allarme ritorna.
				; s, allora entra in routine di servizio
	btfsc	rnflg,tmrun	; conteggio in corso ?
	goto	wselect		; s,allora ignora pressione select
	btfsc	rnflg,alarm	; conteggio terminato ?
	goto	wselect		; s,allora ignora pressione select
	btfsc	rnflg,setmd	; set del tempo in corso ?
	goto	wselect		; s,allora ignora pressione select
				; il timer  in attesa di partire
	movlw	4		; aggiungi 4
	addwf	offset,f	; a offset
	movf	offset,w	;
	sublw	20		;
	btfsc	STATUS,Z	; offset=20 ?
	clrf	offset		; s, allora ricomincia con offset=0
	movf	offset,w	;
	movwf	EEDATA		; memorizza registro offset in EEPROM 00
	movlw	h'00'		;
	movwf	EEADR		;
	bcf	INTCON,GIE	; Disabilita tutti interrupt
	call	writeee		;
	call	ready		;
	bsf	INTCON,GIE	; Abilita tutti interrupt

wselect	btfsc	key,select	; attendi il rilascio
	goto	wselect		; non ancora rilasciato
	call 	dly20		; aspetta 20msec
	btfsc	key,select	; 2 controllo
	goto	select		; continua a controllare
	return	


;-------------------------------------------------------------------------;
;      Questa routine memorizza il tempo del timer nella eepromm          ;
;-------------------------------------------------------------------------;
putee	movlw	h'0C'		; inizializzo il puntatore sui secondi
	btfsc	rnflg,hm	;  attiva la modalit ore-minuti ?
	addlw	2		; s,allora aggiungi 2 al puntatore
	movwf	FSR		; no.Usa l'indirizzamneto indiretto

	movf	INDF,w		; memorizza in eeprom il file register
	movwf	EEDATA		; puntato ed i sucessivi 3.
	movlw	1		
	addwf	offset,w	
	movwf	EEADR
	call	writeee

	incf	FSR,f
	movf	INDF,w
	movwf	EEDATA
	movlw	2		
	addwf	offset,w
	movwf	EEADR
	call	writeee

	incf	FSR,f
	movf	INDF,w
	movwf	EEDATA
	movlw	3		
	addwf	offset,w
	movwf	EEADR
	call	writeee

	incf	FSR,f
	movf	INDF,w
	btfsc	rnflg,hm
	addlw	128
	movwf	EEDATA
	movlw	4		
	addwf	offset,w
	movwf	EEADR
	call	writeee
	return

;-------------------------------------------------------------------------;
;             Questa routine scrive un byte nella EEPROM                  ;
;-------------------------------------------------------------------------;
writeee	bank1			; set up EEADR and EEDATA first
        clrf	EECON1		;                
	bsf	EECON1,WREN	; enable write
 	movlw	h'55'		; magic sequence
  	movwf	EECON2		;               
	movlw	h'AA'		;                  
	movwf	EECON2		;               
	bsf	EECON1,WR	;              
eewloop	btfsc	EECON1,WR	; wait for WR to go low
	goto	eewloop		; not yet
	bsf	EECON1,WREN  	;          
	bcf	EECON1,EEIF	; clear the interrupt flag
	bank0			; return to page 0
	return 


;-------------------------------------------------------------------------;
;		Routine di fine conto alla rovescia
;   Routine eseguita qunado il timer finisce il conto alla rovescia      
;-------------------------------------------------------------------------;
alarm_on
	bsf	rnflg,alarm	; abilita bit flag alarm
	out_off			; disattiva bjt dell'uscita
	bcf	rnflg,tmrun	; ferma conteggio
	return

;-------------------------------------------------------------------------;
;   Questa routine controlla se il timer deve passare dalla modalit
;              ore-minuti alla modalit minuti-secondi
; 	Verifica se il registro hour10 e hour sono entrambi 0
; in caso affermativo esegue clear del bit flag hm del registro rnflg.
;-------------------------------------------------------------------------;


hmtoms	btfss	rnflg,hm	; timmer in modalit hhmm ?
	return			; no,  modalit mmss, ritorna
	movf	hour10,w	; s, carica in W le decine di ore
	andlw 	01111111b	; maschera il bit pi significativo
	iorwf	hour,w		; metto in or con le unit di ore
	btfss	STATUS,Z	; hour e hour10 entrambe = 0 ?
	return			; no, continua con modalit hhmm
	bcf	rnflg,hm	; s, allora setta modalit mmss
	return

;-------------------------------------------------------------------------;                
;      Questa Routine  eseguita quando  rilevata la mancanza 
;           dell'alimentazione dalla rete elettrica.
;-------------------------------------------------------------------------;
blckout	call	dly20		; aspetta un poco
	btfss	key,psctrl	; controlla nuovamente bit flag psctrl
	return			; falso allarme, ritorna.
	nop			; manca l'energia elettrica:i
	bcf	INTCON,T0IE	; disabilita TMR0 interrupts
	clrf	key		; azzera registro ingressi
	movlw	11100000b	; pone basse le basi dei bjt del display
	movwf	PORTA		; e abassa l'uscita quindi
	movlw	0		; spegni tutti i segmenti
	movwf	PORTB		; in modo da ridurre consumo.
	bank1			; change to page 1
        bsf	TRISB,psctrl	; cambia in ingresso linea psctlr di PORTB
        bank0			; change to page 0
	nop			;	
	bsf	INTCON,INTE	; enable RB0/INT interrupt
	sleep			; enter stand by mode
	nop
	nop			; zzz risveglio su RB0/INT interrupt
	nop
	bsf	INTCON,T0IE	; riabilita TMR0 interrupts
	return

;-------------------------------------------------------------------------;
;  The following are various delay routines based on instruction length.  ;  
;  The instruction length is assumed to be 1 microsecond (4Mhz crystal).  ;
;-------------------------------------------------------------------------;
dly20	movlw	20		; delay for 20 milliseconds
                		;*** N millisecond delay routine ***
nmsec	movwf	cntmsec		; delay for N (in W) milliseconds
msecloop
	movlw	248		; load takes 1 microsec
	call	micro4		; by itself CALL takes ...
                                ; 2 + 247 X 4 + 3 + 2 = 995
	nop                 	; 1 more microsec 
	decfsz	cntmsec,f	; 1 when skip not taken, else 2
	goto	msecloop	; 2 here: total 1000 per msecloop
	return			; final time through takes 999 to here
				; overhead in and out ignored

		                ;***  1 millisecond delay routine ***
onemsec	movlw	249		; 1 microsec for load W
				; loops below take 248 X 4 + 3 = 995
micro4	addlw	h'55'		; subtract 1 from 'W'
	btfss	STATUS,Z	; skip when you reach zero
	goto	micro4		; loops takes 4 microsec, 3 for last
	return			; takes 2 microsec
				; call + load  W + loops + return =
				; 2 + 1 + 995 + 2 = 1000 microsec



;
;
;         _\\|//_     
;        (` o o ')
; ______ooO-(_)-Ooo_________
;                             
	END                     
;
; _____.oooO_____Oooo.________
;      (   )     (   )
;       \ (       ) /
;        \_)     (_/
;
;-------------------------------------------------------------------------;
;  		BREVE DESCRIZIONE DEL PROGRAMMA
;-------------------------------------------------------------------------;


; Questo progetto utilizza molte delle caratteristiche del microcontrollore PIC
; * gestione di eventi ad intervalli regolari tramite timer interno
; * utilizzo di alcuni pin in switching come uscite e come ingressi.
; * routine antirimbalzo nella commutazione dei tasti.
; * lettura e scrittura sulla EEPROM interna.
; * utilizzo della modalit sleep con risveglio su interrupt esterno
;	
; Lontano da me l'idea di spiegare il funzionamento dell'intero programma.
; Solo alcune note per cercare di farvi capire a grandi linee,
; e per cercare di ricordarmi, il funzionamento.Il codice  scritto in 
; linguaggio assembler utilizando MPLAB IDE Windows/16 5.31.00.
;
;			COSTANTI
; fuse
; Il valore teorico per fuse  240 ma la deviazione
; in frequenza del quarzo costringe a ritoccare questo valore.
; Sperimentalmente ho trovato buono il valore 235 per il PIC16f84
; alimentato con 5V e 237 per il PIC 16F628 alimmentato a 5V,
; in ambiente fresco,circa 15C.
; C'entra molto anche la temperatura.
;
; swin
; costante che metto nel registro TRISB quando voglio che alcuni
; pin del micro diventino temporaneamente degli ingressi.
;
; lngprss
; determina quanto tempo bisogna tenere premuto un tasto
; prima di entrare nel conteggio veloce durante set del tempo.
;
; spdst	
; speed set.Determina la velocit di incremento dei dgt
; durante programmazione del tempo.
;
;			BIT FLAG
; Di seguito l'elenco, non ordinato, dei bit del registro rnflg
; utilizzati nel programma come flag per definire lo stato del timer.
;
;
; adj
; Il bit flag adj (adjust)  settato ogni 122 interrupt di TMR0 quindi
; ogni 122*4096=499712 us.
; Alla sucessiva esecuzione del interrupt handler  eseguita la sola
; routine adjust per ottenere il sucessivo interrupt handler dopo 288 us.
;
; hm
; Il bit flag hm  settato per segnalare al programma che il timer 
;  nella modalit ore-minuti e quindi occorre gestire il dot.
;
; tmrun
; Il bit flag tmrun (timer run)  settato quando il timer  attivo cio 
; in corso il conto alla rovescia.tmrun rimane settato anche quando
; il timer  posto in pausa.
 
; tmrp
; Il bit flag tmrp (timer paused) che pone il mette in pausa il timer.
; Il timer entra in pausa solo se  attivo.
;
; setmd
; Il bit flag setmd (set mode) segnala al programma che il timer  in 
; set mode cio  in corso la programmazione del tempo iniziale.

; alarm
; Il bit flag alarm  settato quando il timer ha terminato il conteggio
; alla rovescia.E' resettato quando si fa reinezializzare il timer
;
;			FILE REGISTER
; key
; Del registro key sono utilizzati solo i bit mode,go e psctlr.
; I bit sono settati quando sono trovati attivi i
; rispettivi ingressi cio e stato premuto un tasto o  stata
; rilevata l'assenza dell'alimentazione Vcc.

; dgtcnt
; Del registro dgtcnt (digit counter)sono utilizzati solo i primi 2 bit.
; E'utilizzato come contatore modulo 4 per memorizare quale display
; si st rinfrescando

; eetemp
; registro temporaneo usato nella lettura e scrittura di dati sulla eeprom

; tempa, tempb
; registri utilizzati per memorizzare in modo temporaneo lo stato
; delle porte del Pic.Il nuovo valore da mettere sulle porte  
; elaborato prima su questi registri e sucessivamente copiato sulle
; rispettive porte.

; lcnt
; registro loop counter. E'utilizzato come contatore nei cicli.

; intcnt
; interrupt counter.Fle register utilizzato come contaore del numero
; di interrupt causati da overlfow di TMRO.

; rnflg
; Registro i cui bit sono utilizzati come flag per definire lo stato di
; funzionamento del timer.

; pntr
; registro utilizzato come puntatore (pointer) ai registri che contengono
; il tempo.

; flash
; registro utilizzato per la gestione del lampeggio dei displayu e del dot
; settando in primi 4 bit si ottiene il lampeggio delle corrispondenti cifre 
; dei display.Settando il bit dot si ha il lampeggio del dot.
;
; w_tmp, s_tmp, fsr_tmp
; registri utilizzati per ripristinare lo stato del micro all'uscita
; dagli interupt handler.
;
;			MEZZO SECONDO
; 
; Di seguito spiego come il programma esegue la routine halfsec ogni mezzo
; secondo.
; La frequenza del quarzo  4MHz.Il clock interno del Pic  quindi di 1MHz.
; La durata di ogni periodo di clock interno del micro  1us.
; Il prescler  associato a TMR0.IL prescaler rate  1:16.
; Quindi TMR0 incrementa ogni 16 us e l'overflow di TMRO avviene ogni
; 16*256=4096us.
; Nell'interrupt handler di TMR0  decrementato il regitro intcnt,
; inizializzato a 122.
; Quando incnt arriva a zero, quindi dopo 122*4096=499712 us  settato
; il bit flag adj.
; Al sucessivo interrupt adj  rilevato settato ed  invocata la sola
; routine adjust, non  eseguito refresh del display.
; La routine adjust scrive il valore FUSE su TMR0 in modo che questo 
; vada in overflow (passaggio da FFh a 00h) esattamente  dopo 288us.
; Per ottenere  288us TMR0 deve eseguire 288/16=18 incrementi quindi regolo
; TMR0 scrivendoci FUSE=256-18=238.
; 
; Es.
; Dall'istante in cui ha effetto la scrittura di 255d su TMR0, trascorrono 16us
; (presclaler 1:16) prima che diventi TMR0=00d ("d" decimale, base10).
; Questo vuol dire che scrivendo 
; 255 su TMR0 ottengo l'interrupt dopo 16us
; 254 su	TMR0 ottengo l'interrupt dopo 2*16=32us
; (256-n) su TMR0 ottengo l'interrupt dopo n*16 us
;
; RITARDI INTRINSECI 
; 1) il prescaler  azzerato ogni volta che si scrive su TMR0.
; 2)Quando scrivo un valore su TMR0 la scrittura diventa effettiva solo
; dopo un ciclo macchina e il conteggio di TMR0  inibito per i due 
; cicli macchina sucessivi, per consentire il ripristino della
; sincronizzazione con il prescaler.
; 3)Quando TMR0 va in overflow  generato l'interrupt ma ci sono due 
; cicli Dummy prima che il micro esegua l'istruzione all'indirizzo 0004h.
; Maggiori chiarimenti li trovate a pag.28 nel documento DS30430C.pdf
;(data sheet PIC 16F8x) sul sito www.microchip.com.
;
; Quindi l'effettiva scrittura su TMR0 nella ISR
; (Interrupt ServiceROutine oppure Interrupt handler) avviene
; solo dopo una serie di ritardi.
; Via software, giocando sulla durata della istrruzioni,
; cerco di fare in mmodo che questo ritardo diventi 16us.
; In questomodo posso compensare questo ritardo
; scrivendo su TMR0 il numero 238+1=239.
; Infatti l'ISR relativa a TMR0
;  costruita in modo che dall'inizio dell'interrupt 
; all'istante in cui TMR0 ritona ad essere in sincronismo col prescaler
; trascorre un intervallo che  multiplo intero della freq. del periodo
; di conteggio del prescaler.
; Il conteggio dei us trascorsi  basato sulla durata delle istruzioni
; e sulle considerazioni sui ritardi descritte nella nota precedente.
;
; Il ritardo dall'intrrupt di tmr0 alla sua scrittura  di 16us.
; Compenso allora questo ritardo sulla scrittura di TMR0,
; scrivendoci un numero maggiore 238+1=239.
;
; TMR0   [0,1,2,...,255] [0,1,2,...,255] ... [0,1,2,...,255] [->(16us),239,240,...,255]
; intcnt      122              121                 0
; adj          0                0                  1
; 
;       [<---4096us-->] [<---4096us-->] ... [<---4096---->] [ 16+(256-239)*16= 288us ]
;  
;
;
;  			DEVIAZIONE DI FREQUENZA
; Con un quarzo  possibile che dipende molto 
; poco dal circuito sul quale  applicato il quarzo stesso.
; Questo per non vuol dire che la frequenza di oscillazione sia esattamnete 
; quella nominale del quarzo.La lavorazione meccanica del cristallo, per quanto
; precisa, determina una tolleranza sul valore nominale della frequenza
; di risonanza.La temperatura influisce poi sulla frequenza di oscillazione
; del cristallo.
; Dai data sheet dei quarzi da 4MHz si vede che a 25C +-2C i quarzi hanno una 
; tolleranza che va da 5ppm a 100ppm.
; A questa deviazione dal valore nominale occorre aggiungere
; la stabilit in frequenza del quarzo rispetto la temperatura di lavoro.
; Nell'intervallo da 0C a 50C la devizione di frequenza causata
; dalla temperatura pu variare da 10ppm a 200ppm.
; La deviazione dalla freq. nominale rispetto la temperatura 
; pu essere positiva o negativa a seconda del tipo di quarzo.
;
; Suppongo il quarzo abbia una deviazione di frequenza di 100ppm.
; Il periodo del clock interno al Pic  soggetto allo stesso errore relativo
; quindi vale 1us +-0.0001 us.
; Se lascio funzionare il timer per 10 ore ovvero per
; 0ore*3600seocndi=36000 secondi
; l'errore di 100ppm  da una deviazione di 
; 100ppn (36000/1000000)*100 =3.6sec.
; Per compensare questa deviazione di frequenza basta cambiare il valore 
; della costante fuse all'inizio del programma.
; Il valore della costante fuse  quello che vine scritto su TMR0 nella
; routine adjust, per ottenere 1/2 secondo esatto.
; Aumentando il valore di fuse il timer v pi veloce perch TMR0 arriva
; pi presto a 255 e quindi all'overflow.
; Ogni unit di fuse fa variare il timer di 16 us ogni mezzo secondo quindi
; incide per 32 ppm sulla velocit di conteggio.
; Non  possibile eseguire calibrazioni + precise se non aggiungendo
; delle nop prima della scrittura su tmr0.Queste nop introducono un ritardo
; di 1us ognimezzo secondo quindi indidono per 2ppm sulla velocit di conteggio.
; Il valore teorico di fuse,  come detto sopra,  239.
; In pratica occorre aggiustare il valore di fuse.
; Personalmente con quarzi da 4MHz e capacit da 27 pF a 15C ho 
; trovato ideale fuse=236 con un PIC 16F84 e 238 con un PIC 16F628,
; usando sostanzialmente lo stesso codice.
; Notare che la temperatura influenza la frequenza di oscillazione del
; quarzo, anche nella misura di alcune decine di ppm.
; Quini non serve pi di tanto insistere nella precisione.
; Rompersi la testa per spaccare il microsecondo pu
; essere vanificato da una banale variazione di temperatura.

; APPROFONDIMENTI
; * Data sheet dei quarzi a 4MHz serie HC49 sul sito http://www.rs-components.it/
; * Documento PICmicro MID-RANGE MCU FAMILY sul sito  http://www.microchip.com
;   sezione "TMR0 PRESCALER" e "Design Tips".
; * Application notes dal sito http://www.microchip.com
;   AN588 progetto dello stadio oscillatore.
;   AN590 clock design con display led
; * http://www.piclist.com/techref/microchip/cntdn/index.htm
; 
; 			MULTIPLEX DEI PIN 
; I pin RB6,RB7,RB0 del PIC sono gestiti in multiplex.Durante l'esecuzione
; del programma queste linee sono utilizzate prevalentemente come uscite
; per pilotare i alcuni segmenti del display.
; L'idea  scopiazzata dall'application note AN590,
; che trovate sempre sul sito della Microchip.
; Solo durante l'esecuzione dell'interrupt handler di TMR0,quindi ogni
; 4096us,  queste linee sono commutate momentaneamente come ingressi.
; I valori letti su queste linee sono memorizzati nel file register key.
; Quando il micro h terminato l'esecuzione dell'interrupt handler di
; di tmr0 simette a eseguire il loop principale (main).
; Nel main loop il micro  non fa altro che controllarelo stato dei
; bit del registro key.Come detto lo stato di questi bit  aggiornato
; ogni 4096 usec circa.
; Quando trova un bit di key settato allora chiama la relativa routine
; di servizio.
; All'inizio delle routine di servizio c' un doppio controllo
; sull'effettivo stato dei bit del registro key.
; La doppia verifica dello stato del bit nel registrokey  avviene
; con l'interposizione di un intervallo di 20msec, per evitare i
; rimbalzi dei tasti.
; Nel caso sia superato il test  eseguita la relativa routine.
; La stessa routine termina quando  superato il doppio test di rilasciao del tasto.
;
; Alcune routine sono riprese dal progetto seguente 
; http://www.piclist.com/techref/microchip/cntdn/index.htm
; 
;
;			TEMPO MEMORIZZATO IN EPROM
; Nelle prime 4 celle della eeprom sono memorizzati i valori
; che compaiono sul display all'accensione del timer e/o alla sua
; reinizializzazione.Nel caso il tempo memorizzato rappresenti ore-minuti
; allora  settato il bit 7 (MSB) della cella eeprom.
; 
; Nella cella eeprom 0 memorizzo offset per scelta dei tempi
; Il File register offset mantiene valore letto dalla celle eeprom 0
;
;
; EEPROM
; eeprom addr
; 0	offset
; 1	time1a		<- offset=0
; 2	time1b
; 3     time1c
; 4 	time1d
; 5	time2a		<- offset=4
; 6	time2b
; 7     time2c
; 8 	time2d
; ecc..                           8,12,16
;
;
;
; GENERAL PURPOUSE FILE REGISTER LOCATION (DATA MEMORY)
;
; 20 	sec
; 21	sec10
; 22	min
; 23	min10
; 24	hour
; 25	hounr10
